@@ -27,6 +27,7 @@
 #include <ggv-prefs.h>
 #include <ggv-prefs-ui.h>
 #include <ggv-sidebar.h>
+#include <cursors.h>
 
 #include <bonobo.h>
 
@@ -127,14 +128,6 @@ get_status(GgvPostScriptView *ps_view, BonoboArg *arg)
 	*(CORBA_string *)arg->_value = CORBA_string_dup(status);
 }
 
-static void
-prefs_changed_handler(gpointer data)
-{
-	GgvPostScriptView *ps_view = GGV_POSTSCRIPT_VIEW(data);
-
-	/* TODO: set sidebar properties - coordinate units */
-}
-
 static void
 get_title(GgvPostScriptView *ps_view, BonoboArg *arg)
 {
@@ -153,6 +146,19 @@ get_title(GgvPostScriptView *ps_view, BonoboArg *arg)
 	*(CORBA_string *)arg->_value = CORBA_string_dup(title);
 }
 
+static void
+set_file_items_sensitivity(GgvPostScriptView *ps_view, gboolean sens)
+{
+	gchar *prop_val = sens?"1":"0";
+
+	bonobo_ui_component_set_prop(ps_view->priv->uic, "/commands/FileSaveMarked",
+                                     "sensitive", prop_val, NULL);
+	bonobo_ui_component_set_prop(ps_view->priv->uic, "/commands/FilePrintMarked",
+                                     "sensitive", prop_val, NULL);
+	bonobo_ui_component_set_prop(ps_view->priv->uic, "/commands/FilePrintAll",
+                                     "sensitive", prop_val, NULL);
+}
+
 static void
 _set_page_item_sensitivity(GgvPostScriptView *ps_view, BonoboUIComponent *uic)
 {
@@ -219,6 +225,8 @@ notify_page_change(GgvPostScriptView *ps_view)
 	BonoboArg *arg;
 
 	set_page_item_sensitivity(ps_view);
+	ggv_sidebar_page_changed(ps_view->priv->sidebar,
+							 gtk_gs_get_current_page(GTK_GS(ps_view->priv->gs)));
 
 	arg = bonobo_arg_new(TC_CORBA_long);
 	*(CORBA_long *)arg->_value = gtk_gs_get_current_page(GTK_GS(ps_view->priv->gs));
@@ -272,7 +280,9 @@ load_ps(GgvPostScriptView *ps_view, const gchar *fname)
 	notify_page_count_change(ps_view);
 	notify_orientation_change(ps_view);
 	ggv_postscript_view_goto_page(ps_view, 0);
+	notify_page_change(ps_view);
 	ggv_sidebar_update_coordinates(ps_view->priv->sidebar, 0.0, 0.0);
+	set_file_items_sensitivity(ps_view, ggv_postscript_view_get_current_page(ps_view) >= 0);
 }
 
 /*
@@ -347,8 +357,8 @@ view_button_press_cb(GtkWidget *widget, GdkEventButton *event, gpointer data)
 {
 	GgvPostScriptView *ps_view;
 
-	g_return_if_fail(data != NULL);
-	g_return_if_fail(GGV_IS_POSTSCRIPT_VIEW(data));
+	g_return_val_if_fail(data != NULL, FALSE);
+	g_return_val_if_fail(GGV_IS_POSTSCRIPT_VIEW(data), FALSE);
 
 	ps_view = GGV_POSTSCRIPT_VIEW(data);
 
@@ -363,7 +373,7 @@ view_button_press_cb(GtkWidget *widget, GdkEventButton *event, gpointer data)
 
 		ps_view->priv->pan = TRUE;
 		if(pan_cursor == NULL)
-			pan_cursor = gdk_cursor_new(GDK_FLEUR);
+			pan_cursor = cursor_get(widget->window, CURSOR_HAND_CLOSED);
 			
 		gtk_grab_add(widget);
 		gdk_pointer_grab(widget->window, FALSE,
@@ -384,8 +394,8 @@ view_button_release_cb(GtkWidget *widget, GdkEventButton *event, gpointer data)
 {
 	GgvPostScriptView *ps_view;
 
-	g_return_if_fail(data != NULL);
-	g_return_if_fail(GGV_IS_POSTSCRIPT_VIEW(data));
+	g_return_val_if_fail(data != NULL, FALSE);
+	g_return_val_if_fail(GGV_IS_POSTSCRIPT_VIEW(data), FALSE);
 
 	ps_view = GGV_POSTSCRIPT_VIEW(data);
 
@@ -406,8 +416,8 @@ view_motion_cb(GtkWidget *widget, GdkEventMotion *event, gpointer data)
 {
 	GgvPostScriptView *ps_view;
 
-	g_return_if_fail(data != NULL);
-	g_return_if_fail(GGV_IS_POSTSCRIPT_VIEW(data));
+	g_return_val_if_fail(data != NULL, FALSE);
+	g_return_val_if_fail(GGV_IS_POSTSCRIPT_VIEW(data), FALSE);
 
 	ps_view = GGV_POSTSCRIPT_VIEW(data);
 	if(ps_view->priv->pan) {
@@ -594,7 +604,7 @@ ggv_postscript_view_get_ps(GgvPostScriptView *ps_view, gint *active_rows)
 {
 	gchar *ps;
 
-	if(active_rows == NULL)
+	if(!active_rows || active_rows[0] == -1)
 		ps = gtk_gs_get_postscript(GTK_GS(ps_view->priv->gs), NULL);
 	else {
 		gint num, i;
@@ -672,6 +682,28 @@ save_file_sel_ok_clicked(GtkWidget *widget, gpointer data)
 		/* get file name */
 		gtk_widget_hide(ps_view->priv->save_file_sel);
 		fname = gtk_file_selection_get_filename(GTK_FILE_SELECTION(ps_view->priv->save_file_sel));
+
+		if(ggv_file_readable(fname)) {
+			GtkWidget *dlg;
+		
+			dlg = gtk_message_dialog_new(GTK_WINDOW(gtk_widget_get_toplevel(ps_view->priv->gs)),
+										 GTK_DIALOG_MODAL,
+										 GTK_MESSAGE_QUESTION,
+										 GTK_BUTTONS_YES_NO,
+										 _("A file with this name already exists.\n"
+										   "Do you want to overwrite it?"));
+			gtk_widget_show(dlg);
+			switch(gtk_dialog_run(GTK_DIALOG(dlg))) {
+			case GTK_RESPONSE_NO:
+			case GTK_RESPONSE_DELETE_EVENT:
+				gtk_widget_destroy(dlg);
+				g_free(doc);
+				return;
+			default:
+				break;
+			}
+			gtk_widget_destroy(dlg);
+		}
 		
 		if(fname && (f = fopen(fname, "w")) != NULL) {
 			fwrite(doc, 1, strlen(doc), f);
@@ -740,6 +772,13 @@ verb_FileSaveMarked(BonoboUIComponent *uic, gpointer data, const char *cname)
 	if(!GTK_WIDGET_VISIBLE(ps_view->priv->save_file_sel)) {
 		gtk_widget_set_usize(GTK_WIDGET(ps_view->priv->save_file_sel),
 							 file_sel_width, file_sel_height);
+		if(GTK_GS(ps_view->priv->gs)->gs_filename) {
+			gchar *path = g_path_get_dirname(GTK_GS(ps_view->priv->gs)->gs_filename);
+			gchar *path2 = g_strconcat(path, "/", NULL);
+			g_free(path);
+			gtk_file_selection_set_filename(GTK_FILE_SELECTION(ps_view->priv->save_file_sel), path2);
+			g_free(path2);
+		}
 		gtk_widget_show(ps_view->priv->save_file_sel);
 		ggv_raise_and_focus_widget(ps_view->priv->save_file_sel);
 	}
@@ -929,10 +968,12 @@ ggv_postscript_view_create_ui(GgvPostScriptView *ps_view)
 	bonobo_ui_util_set_ui(ps_view->priv->uic, DATADIR,
 						  "ggv-postscript-view-ui.xml", 
 						  "GGV", NULL);
-	bonobo_ui_component_object_set(ps_view->priv->uic, "/Sidebar/GgvSidebar",
-								   BONOBO_OBJREF(ps_view->priv->sidebar), NULL);
+	bonobo_ui_component_set_prop(ps_view->priv->uic, "/Sidebar",
+								 "placement", ggv_right_panel?"right":"left", NULL);
 	bonobo_ui_component_set_prop(ps_view->priv->uic, "/Sidebar",
 								 "hidden", ggv_panel?"0":"1", NULL);
+	bonobo_ui_component_object_set(ps_view->priv->uic, "/Sidebar/GgvSidebar",
+								   BONOBO_OBJREF(ps_view->priv->sidebar), NULL);
 	bonobo_ui_component_add_verb_list_with_data(ps_view->priv->uic,
 												ggv_postscript_view_verbs,
 												ps_view);
@@ -1456,8 +1497,8 @@ gfloat
 ggv_postscript_view_zoom_to_fit(GgvPostScriptView *ps_view,
 								gboolean fit_width)
 {
-	g_return_if_fail(ps_view != NULL);
-	g_return_if_fail(GGV_IS_POSTSCRIPT_VIEW(ps_view));
+	g_return_val_if_fail(ps_view != NULL,0.0);
+	g_return_val_if_fail(GGV_IS_POSTSCRIPT_VIEW(ps_view),0.0);
 
 	/* we don't know how not to keep the aspect ratio */
 	return gtk_gs_zoom_to_fit(GTK_GS(ps_view->priv->gs), fit_width);
@@ -1532,8 +1573,6 @@ ggv_postscript_view_destroy(BonoboObject *object)
 		ps_view->priv->uic = NULL;
 	}
 
-	ggv_prefs_remove_callback(ps_view);
-
 	if (BONOBO_OBJECT_CLASS (ggv_postscript_view_parent_class)->destroy)
 		BONOBO_OBJECT_CLASS(ggv_postscript_view_parent_class)->destroy(object);
 }
@@ -1599,11 +1638,6 @@ ggv_postscript_view_class_init(GgvPostScriptViewClass *klass)
 {
 	GObjectClass *object_class = (GObjectClass *)klass;
 	BonoboObjectClass *bonobo_object_class = (BonoboObjectClass *)klass;
-	gint n = gtk_gs_defaults_get_paper_count(), i;
-	BonoboUINode *placeholder;
-	BonoboUINode *item;
-	gchar *enc_str;
-	GtkGSPaperSize *papersizes;
 	POA_GNOME_GGV_PostScriptView__epv *epv;
 
 	klass->priv = g_new0(GgvPostScriptViewClassPrivate, 1);
@@ -1689,6 +1723,19 @@ property_control_get_cb(BonoboPropertyControl *property_control,
 #endif
 }
 
+static void
+document_changed_cb(GtkGS *gs, gpointer data)
+{
+	GgvPostScriptView *ps_view = GGV_POSTSCRIPT_VIEW(data);
+	gchar *fname;
+	gint page = gtk_gs_get_current_page(GTK_GS(ps_view->priv->gs));
+
+	fname = g_strdup(GTK_GS(ps_view->priv->gs)->gs_filename);
+	gtk_gs_load(GTK_GS(ps_view->priv->gs), fname);
+	ggv_postscript_view_goto_page(ps_view, page);
+	g_free(fname);				
+}
+
 static void
 interpreter_message_cb(GtkGS *gs, gchar *msg, gpointer data)
 {
@@ -1710,6 +1757,16 @@ interpreter_error_cb(GtkGS *gs, gint status, gpointer data)
 	notify_page_count_change(GGV_POSTSCRIPT_VIEW(data));
 }
 
+static void
+view_realized_cb(GtkGS *gs, gpointer data)
+{
+	GdkCursor *cursor;
+
+	cursor = cursor_get(GTK_WIDGET(gs)->window, CURSOR_HAND_OPEN);
+	gdk_window_set_cursor(GTK_WIDGET(gs)->window, cursor);
+	gdk_cursor_destroy(cursor);
+}
+
 GgvPostScriptView *
 ggv_postscript_view_construct(GgvPostScriptView *ps_view,
 							  GtkGS *gs, gboolean zoom_fit)
@@ -1741,8 +1798,13 @@ ggv_postscript_view_construct(GgvPostScriptView *ps_view,
 					 G_CALLBACK(interpreter_message_cb), ps_view);
 	g_signal_connect(G_OBJECT(gs), "interpreter_error",
 					 G_CALLBACK(interpreter_error_cb), ps_view);
+	g_signal_connect(G_OBJECT(gs), "document_changed",
+					 G_CALLBACK(document_changed_cb), ps_view);
+	g_signal_connect(G_OBJECT(gs), "realize",
+					 G_CALLBACK(view_realized_cb), NULL);
 
 	gtk_widget_show(GTK_WIDGET(gs));
+
 	ps_view->priv->zoom_fit = zoom_fit;
 
 	/* Property Bag */
@@ -1823,8 +1885,6 @@ ggv_postscript_view_construct(GgvPostScriptView *ps_view,
 
 	ps_view->priv->sidebar = ggv_sidebar_new(ps_view);
 
-	ggv_prefs_add_callback(prefs_changed_handler, ps_view);
-
 	return ps_view;
 }
 
