@@ -144,10 +144,6 @@ control_property_changed_handler(BonoboListener    *listener,
                                         BONOBO_ARG_GET_STRING(any));
         }
         else if(!g_strcasecmp(event_name, "Bonobo/Property:change:page")) {
-                GtkTreeSelection *sel;
-                gchar path_str[16];
-                GtkTreePath *path;
-
                 if(win->current_page == BONOBO_ARG_GET_INT(any) &&
                    win->current_page < 0)
                         return;
@@ -251,8 +247,10 @@ static void
 verb_FileReload(BonoboUIComponent *uic, gpointer data, const char *cname)
 {
         GgvWindow *win = GGV_WINDOW(data);
+        gint page = win->current_page;
 
         ggv_window_load(win, win->filename);
+        bonobo_pbclient_set_long(win->pb, "page", page, NULL);
 }
 
 static void
