@@ -342,7 +342,7 @@ gtk_gs_class_init (GtkGSClass *klass)
                        "interpreter_message",
                        G_TYPE_FROM_CLASS(object_class),
                        G_SIGNAL_RUN_LAST,
-                       GTK_SIGNAL_OFFSET(GtkGSClass, interpreter_message),
+                       G_STRUCT_OFFSET(GtkGSClass, interpreter_message),
                        NULL,
                        NULL,
                        ggv_marshaller_VOID__POINTER,
@@ -352,13 +352,24 @@ gtk_gs_class_init (GtkGSClass *klass)
                        "interpreter_error",
                        G_TYPE_FROM_CLASS(object_class),
                        G_SIGNAL_RUN_LAST,
-                       GTK_SIGNAL_OFFSET(GtkGSClass, interpreter_error),
+                       G_STRUCT_OFFSET(GtkGSClass, interpreter_error),
                        NULL,
                        NULL,
                        ggv_marshaller_VOID__INT,
                        G_TYPE_NONE,
                        1, G_TYPE_INT);
 
+	gtk_gs_signals[DOCUMENT_CHANGED] = g_signal_new(
+                       "document_changed",
+                       G_TYPE_FROM_CLASS(object_class),
+                       G_SIGNAL_RUN_LAST,
+                       G_STRUCT_OFFSET(GtkGSClass, document_changed),
+                       NULL,
+                       NULL,
+                       g_cclosure_marshal_VOID__VOID,
+                       G_TYPE_NONE,
+                       0);
+
         object_class->destroy = gtk_gs_destroy;
 
         widget_class->realize       = gtk_gs_realize;
@@ -1843,7 +1854,7 @@ gtk_gs_next_page(GtkGS *gs)
 	event.xclient.message_type  = gdk_x11_atom_to_xatom(gs_class->next_atom);
 	event.xclient.format        = 32;
         
-        gdk_send_xevent(gs->message_window, FALSE, 0, &event);     
+	_gdk_send_xevent(gdk_display,gs->message_window, FALSE, 0, &event); 
  
         gdk_flush();
 
@@ -1965,7 +1976,8 @@ gtk_gs_set_page_size(GtkGS *gs, gint new_pagesize, gint pageid)
         g_return_val_if_fail(GTK_IS_GS(gs), FALSE);
 
         if(new_pagesize == -1) {
-                new_pagesize = gs->default_size;
+                if(gs->default_size > 0)
+                        new_pagesize = gs->default_size;
                 if (!gs->override_size && gs->doc) {
                         /* If we have a document:
                            We use -- the page size (if specified)
@@ -2002,7 +2014,7 @@ gtk_gs_set_page_size(GtkGS *gs, gint new_pagesize, gint pageid)
                 new_urx = papersizes[new_pagesize].width;
                 new_ury = papersizes[new_pagesize].height;
         }
-        else if ((gs->doc->epsf || new_pagesize == -1)) { /* epsf or bbox */
+        else if ((gs->doc->epsf && !gs->override_size) || new_pagesize == -1) { /* epsf or bbox */
                 if((gs->doc->pages) &&
                    (gs->doc->pages[pageid].boundingbox[URX] > gs->doc->pages[pageid].boundingbox[LLX]) &&
                    (gs->doc->pages[pageid].boundingbox[URY] > gs->doc->pages[pageid].boundingbox[LLY])) {
@@ -2034,6 +2046,11 @@ gtk_gs_set_page_size(GtkGS *gs, gint new_pagesize, gint pageid)
                 }
         }
 
+        if(new_urx <= new_llx)
+                new_urx = papersizes[8].width;
+        if(new_ury <= new_lly)
+                new_ury = papersizes[8].height;
+
         /* If bounding box changed, setup for new size. */
                 /* gtk_gs_disable_interpreter (gs); */
         if ((new_llx != gs->llx) || (new_lly != gs->lly) ||
