@@ -61,18 +61,28 @@ main(int argc, char **argv)
 	CORBA_Environment ev;
 	CORBA_ORB orb;
 	GError *error;
-	poptContext *ctx;
+	poptContext ctx;
+	GValue value = { 0, };
+	GnomeProgram *program;
 
-	bindtextdomain(PACKAGE, GNOMELOCALEDIR);
-	bind_textdomain_codeset(PACKAGE, "UTF-8");
-	textdomain(PACKAGE);
+	bindtextdomain(GETTEXT_PACKAGE, GNOMELOCALEDIR);
+	bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
+	textdomain(GETTEXT_PACKAGE);
 
-	ctx = g_new0(poptContext, 1);
-	gnome_init_with_popt_table("Gnome Ghostview", VERSION,
-                                   argc, argv, NULL, 0, ctx);
+	program = gnome_program_init("ggv", VERSION,
+                                     LIBGNOMEUI_MODULE,
+                                     argc, argv,
+                                     GNOME_PARAM_POPT_TABLE, 0,
+                                     GNOME_PARAM_APP_DATADIR, DATADIR, NULL);
 
 	CORBA_exception_init(&ev);
 
+        g_value_init(&value, G_TYPE_POINTER);
+	g_object_get_property(G_OBJECT(program), GNOME_PARAM_POPT_CONTEXT,
+                              &value);
+	ctx = g_value_get_pointer(&value);
+	g_value_unset(&value);
+
 	error = NULL;
 	if (gconf_init(argc, argv, &error) == FALSE) {
 		g_assert(error != NULL);
@@ -84,7 +94,7 @@ main(int argc, char **argv)
 	if (bonobo_ui_init ("Gnome Ghostview", VERSION, &argc, argv) == FALSE)
 		g_error (_("Could not initialize Bonobo!\n"));
 
-        gtk_idle_add (create_windows_from_cmdline, ctx);
+        gtk_idle_add (create_windows_from_cmdline, &ctx);
 
         ggv_prefs_add_callback((GgvPrefsCb)ggv_window_apply_prefs, NULL);
 
